@@ -335,10 +335,8 @@ class tensor(object):
         if isinstance(other, ttb.tensor):
             if self.shape != other.shape:
                 assert False, 'Inner product must be between tensors of the same size'
-            x = np.reshape(self.data, (self.data.size,))
-            y = np.reshape(other.data, (other.data.size,))
-            #x = np.reshape(self.data, (1, self.data.size))
-            #y = np.reshape(other.data, (other.data.size, 1))
+            x = np.reshape(self.data, (self.data.size,), order='F')
+            y = np.reshape(other.data, (other.data.size,), order='F')
             return x.dot(y)
         elif isinstance(other, (ttb.ktensor, ttb.sptensor, ttb.ttensor)):
             # Reverse arguments and call specializer code
@@ -722,7 +720,6 @@ class tensor(object):
         # Np transpose does error checking on order, acts as permutation
         return ttb.tensor.from_data(np.transpose(self.data, order))
 
-    # TODO should this be a property?
     def reshape(self, *shape):
         """
         Reshapes a tensor
@@ -771,20 +768,16 @@ class tensor(object):
     def symmetrize(self, grps=None, version=None):
         """
         Symmetrize a tensor in the specified modes
-
         Notes
         -----
         It is *the same or less* work to just call X = symmetrize(X) then to first check if X is symmetric and then
         symmetrize it, even if X is already symmetric.
-
         Parameters
         ----------
         grps
         version
-
         Returns
         -------
-
         """
         n = self.ndims
         sz = np.array(self.shape)
@@ -897,6 +890,100 @@ class tensor(object):
 
             return Y
 
+    def ttm(self, matrix, dims=None, transpose=False):
+        """
+        Tensor times matrix
+
+        Parameters
+        ----------
+        matrix: :class:`Numpy.ndarray`, list[:class:`Numpy.ndarray`]
+        dims: :class:`Numpy.ndarray`, int
+        transpose: boolean
+        """
+
+        if dims is None:
+            dims = np.arange(self.ndims)
+        elif isinstance(dims, list):
+            dims = np.array(dims)
+        elif np.isscalar(dims) or isinstance(dims, list):
+            dims = np.array([dims])
+
+        if isinstance(matrix, list):
+            # Check that the dimensions are valid
+            dims, vidx = ttb.tt_dimscheck(dims, self.ndims, len(matrix))
+            # Calculate individual products
+            Y = self.ttm(matrix[vidx[0]], dims[0], transpose)
+            for k in range(1,dims.size):
+                Y = Y.ttm(matrix[vidx[k]], dims[k], transpose)
+            return Y
+
+        if not isinstance(matrix, np.ndarray):
+            assert False, "matrix must be of type numpy.ndarray"
+
+        if not (dims.size == 1 and np.isin(dims, np.arange(self.ndims))):
+            assert False, "dims must contain values in [0,self.dims]"
+
+        # old version (ver=0)
+        shape = np.array(self.shape)
+        n = dims[0]
+        order = np.array([n] + list(range(0,n)) + list(range(n+1,self.ndims)))
+        newdata = self.permute(order)
+        ids = np.array(list(range(0,n)) + list(range(n+1,self.ndims)))
+        newdata = np.reshape(newdata.data, (shape[n],np.prod(shape[ids])), order="F")
+        if transpose:
+            newdata = matrix.T @ newdata
+            p = matrix.shape[1]
+        else:
+            newdata = matrix @ newdata
+            p = matrix.shape[0]
+
+        newshape = np.array([p] + list(shape[range(0,n)]) + list(shape[range(n+1,self.ndims)]))
+        Y = np.reshape(newdata, newshape, order="F")
+        Y = np.transpose(Y, np.argsort(order))
+        return ttb.tensor.from_data(Y)
+
+    def ttt(self, other, selfdims=None, otherdims=None):
+        """
+        Tensor mulitplication (tensor times tensor)
+
+        Parameters
+        ----------
+        other: :class:`ttb.tensor`
+        selfdims: :class:`Numpy.ndarray`, int
+        otherdims: :class:`Numpy.ndarray`, int
+        """
+
+        if not isinstance(other, tensor):
+            assert False, "other must be of type tensor"
+
+        if selfdims is None:
+            selfdims = np.array([])
+            selfshape = ()
+        else:
+            selfshape = tuple(np.array(self.shape)[selfdims])
+
+        if otherdims is None:
+            otherdims = selfdims.copy()
+            othershape = ()
+        else:
+            othershape = tuple(np.array(other.shape)[otherdims])
+
+        if not selfshape == othershape:
+            assert False, "Specified dimensions do not match"
+
+        # Compute the product
+
+        # Avoid transpose by reshaping self and computing result = self * other
+        amatrix = ttb.tenmat.from_tensor_type(self, cdims=selfdims)
+        bmatrix = ttb.tenmat.from_tensor_type(other, rdims=otherdims)
+        cmatrix = amatrix * bmatrix
+
+        # Check whether or not the result is a scalar
+        if isinstance(cmatrix, ttb.tenmat):
+            return ttb.tensor.from_tensor_type(cmatrix)
+        else:
+            return cmatrix
+
     def ttv(self, vector, dims=None):
         """
         Tensor times vector
@@ -913,6 +1000,8 @@ class tensor(object):
             dims = np.array([dims])
 
         # Check that vector is a list of vectors, if not place single vector as element in list
+        if isinstance(vector, list):
+            return self.ttv(np.array(vector), dims)
         if len(vector.shape) == 1 and isinstance(vector[0], (int, float, np.int_, np.float_)):
             return self.ttv(np.array([vector]), dims)
 
@@ -924,13 +1013,6 @@ class tensor(object):
             if vector[vidx[i]].shape != (self.shape[dims[i]], ):
                 assert False, "Multiplicand is wrong size"
 
-        # TODO: not sure what this special case handles
-        #if exist('tensor/ttv_single', 'file') == 3:
-        #    c = a
-        #    for i = numel(dims): -1: 1
-        #        c = ttv_single(c, v{vidx(i)}, dims(i))
-        #    return c
-
         # Extract the data
         c = self.data.copy()
 
@@ -944,7 +1026,7 @@ class tensor(object):
         sz = np.array(self.shape)[np.concatenate((remdims, dims))]
 
         for i in range(dims.size-1, -1, -1):
-            c = np.reshape(c, tuple([np.prod(sz[0:n-1]), sz[n-1]]))
+            c = np.reshape(c, tuple([np.prod(sz[0:n-1]), sz[n-1]]), order='F')
             c = c.dot(vector[vidx[i]])
             n -= 1
         # If needed, convert the final result back to tensor
@@ -968,7 +1050,7 @@ class tensor(object):
         elif dims > 0:
             assert False, "Invalid modes in ttsv"
 
-        if version is not None:  # Calculate the old way
+        if version == 1:  # Calculate the old way
             P = self.ndims
             X = np.array([vector for i in range(P)])
             if dims == 0:
@@ -978,9 +1060,7 @@ class tensor(object):
             else:
                 return self.ttv(X, -np.arange(1, -dims+1))
 
-        else:  # Calculate the new way
-            if dims != 0:
-                assert False, "New version only support vector times all modes"
+        elif version == 2 or version is None:  # Calculate the new way
             d = self.ndims
             sz = self.shape[0]  # Sizes of all modes must be the same
 
@@ -989,17 +1069,18 @@ class tensor(object):
 
             y = self.data
             for i in range(drem, 0, -1):
-                yy = np.reshape(y, (sz**(dnew + i -1), sz))
+                yy = np.reshape(y, (sz**(dnew + i -1), sz), order='F')
                 y = yy.dot(vector)
 
             # Convert to matrix if 2-way or convert back to tensor if result is >= 3-way
-            # TODO: currently this only support scalar return so these are ignored in coverage
-            if dnew == 2:  # pragma: no cover
-                return np.reshape(y, [sz, sz])
-            elif dnew > 2:  # pragma: no cover
-                return ttb.tensor.from_data(np.reshape(y, sz*np.ones(dnew)))
+            if dnew == 2:  
+                return np.reshape(y, [sz, sz], order='F')
+            elif dnew > 2:  
+                return ttb.tensor.from_data(np.reshape(y, sz*np.ones(dnew, dtype=np.int), order='F'))
             else:
                 return y
+        else:
+            assert False, "Invalid value for version; should be None, 1, or 2"
 
     def __setitem__(self, key, value):
         """
